using System;
using System.Collections.Generic;
using System.Text;

namespace CPIDR
{

  /// <summary>
  /// Static class of methods to take a tagged string and produce 
  /// the information needed for idea density rating.
  /// By use of partial classes, the (long) rule set is stored in
  /// a separate file, IdeaDensityRaterRules.cs.
  /// </summary>
  static partial class IdeaDensityRater
  {

    /// <summary>
    /// Idea density rating of tagged text.
    /// </summary>
    /// <param name="taggedtext">Input string in typical tagger format, such as "the/DT dogs/NNS barked/VBD".</param>
    /// <param name="wordcount">Computed word count.</param>
    /// <param name="propcount">Computed proposition count.</param>
    /// <returns>A WordList structure which can be turned into a string and displayed to show the details of the analysis.</returns>
    public static WordList RateTaggedText(string taggedtext, bool speechMode, out int wordcount, out int propcount)
    {
      WordList w = new WordList(taggedtext);
      w.ApplyIdeaCountingRules(speechMode);

      int wc = 0;
      int pc = 0;
      foreach (WordListItem i in w.items)
      {
        if (i.isword) wc++;
        if (i.isprop) pc++;
      }

      wordcount = wc;
      propcount = pc;
      return w;
    }


    public class WordListItem
    {
      public string token;
      public string tag;
      public bool isword;
      public bool isprop;
      public int rulenumber;

      public WordListItem()   // default constructor
      {
        this.token = "";
        this.tag = "";
        this.isword = false;
        this.isprop = false;
        this.rulenumber = 0;
      }

      public WordListItem(string token, string tag, bool isword, bool isprop, int rulenumber)
      {
        this.token = token;
        this.tag = tag;
        this.isword = isword;
        this.isprop = isprop;
        this.rulenumber = rulenumber;
      }

      public new string ToString()
      {
        string r = 
          String.Format(
            " {2:#000} {3,-4} {1} {0} {4}",    // most amusing format string you've seen lately
            (this.isprop ? 'P' : ' '),
            (this.isword ? 'W' : ' '),
            this.rulenumber,
            this.tag,
            this.token
        );
        if (this.tag == ".")
        {
          return r + "\r\n";    // skip extra line after end of sentence marker
        }
        else
        {
          return r;
        }
      }

    }




    /// <summary>
    /// Data structure: a list of tagged words with various attributes marked.
    /// Initialized with 10 null items at the beginning so that rules can look
    /// back 10 spaces from the current location without running past
    /// the beginning.  More of this class in IdeaDensityRaterRules.cs.
    /// </summary>
    public partial class WordList
    {
      public List<WordListItem> items;

      public WordList()  // default constructor
      {
        this.items = new List<WordListItem>();
        // Start with 10 null items at the beginning
        WordListItem wi = new WordListItem("", "", false, false, 0);
        for (int i = 0; i < 10; i++) this.items.Add(wi);
      }

      /// <summary>
      /// Constructs a WordList from a tagged text string of the
      /// form "token/tag token/tag token/tag".
      /// </summary>
      public WordList(string taggedtext)
        : this()  // makes it call default constructor
      {
        // Split input string at the blanks, giving token/tag, token/tag etc.
        string[] tokentags = taggedtext.Split(' ');

        foreach (string s in tokentags)
        {
          // Find the last slash in token/tag (because token may also contain a slash)
          int i = s.LastIndexOf('/');

          // Create an item using the token and tag
          this.items.Add(
              new WordListItem(s.Substring(0, i).ToLower(),
                                      s.Substring(i + 1),
                                      false,
                                      false,
                                      0)
                              );
        }
      }

      /// <summary>
      /// Renders a WordList as a table suitable for printing in fixed-pitch type.
      /// Each of the booleans is an asterisk or blank for true or false respectively.
      /// Items with empty token are omitted.
      /// </summary>
      public new string ToString()
      {
        StringBuilder b = new StringBuilder();
        foreach (WordListItem i in this.items)
        {
          if (i.token != "") b.Append(i.ToString() + "\r\n");
        }
        return b.ToString();
      }

    }

  }

}


